from csv import reader
from math import inf
from copy import deepcopy
import matplotlib.pyplot as plt


def creationTableau2D(nomFichier):
    """
    lit un fichier csv et crée une liste de listes 
    chaque liste correspond à une ligne du fichier csv
    
    paramètres:
    ----------
    nomFichier : string
                chaine de caractères nommant un fichier .csv
 
    résultat:
    ---------
    list
        une liste de listes (une ligne du fichier .csv)
    """

    tab2D = []
    with open(nomFichier,"r",newline='',encoding='utf-8') as fichier:
        donnees = reader(fichier,delimiter=';')
        for ligne in donnees:
            tab2D.append(ligne)
    return tab2D
    
def mise_en_forme(tab2D):
    """
    transforme la liste de listes venant du fichier iris.csv non exploitable
    pour les calculs en une liste exploitable
    
    paramètres:
    ----------
    tab2D :     list
                
    résultat:
    ---------
    list
        une liste de listes exploitable pour les calculs
    """
    liste2D = []
    liste2D.append(tab2D[0])
    for i in range(1,len(tab2D)):
        liste = []
        liste = tab2D[i][0].split(',')
        for j in range(4):
            liste[j] = float(liste[j])
        liste2D.append(liste)
    return liste2D

def creation_base_apprentissage_et_test(tab2D):
    """
    partage en deux parties égales la liste de listes, une partie
    sera la base d'apprentissage, l'autre la base de test
    
    paramètres:
    ----------
    tab2D :     list
                
    résultat:
    ---------
    list
        une liste de listes constituée de la base d'apprentissage et de la base de test
    """
    base_test = []
    base_app = []
    bases = []
    for i in range(1,len(tab2D)):
        if i % 2 == 0:
            base_test.append(tab2D[i])
        else:
            base_app.append(tab2D[i])
    bases = [base_app,base_test]
    return bases

def dist_eucli(x1,y1,x2,y2):
    return (x1 - x2)**2 + (y1 - y2)**2
    
def dist_manhattan(x1,y1,x2,y2):
    return abs(x1 - x2) + abs(y1 - y2)
    
def ppv(base_app,elements):
    """
    classe les fleurs dans elements à partir de la base base_app
    selon l'algorithme du plus proche voisin
    
    paramètres:
    ----------
    base_app :     list
                la base d'apprentissage
    elements :     list
                 les fleurs à classer
    résultat:
    ---------
    list
        les fleurs ont été classées
    """
    elements_bis = deepcopy(elements)
    for i in range(len(elements)):
        d_min = inf
        b_min = None
        for b in base_app:
            d = dist_eucli(elements[i][2],elements[i][3],b[2],b[3])
            if d < d_min:
                
                d_min = d
                b_min = b
        
        elements_bis[i][4] = b_min[4]
    return elements_bis

def mesure_performance(base_test,classement):
    """
    mesure la performance des prévisions effectuées sur la base_test
    
    
    paramètres:
    ----------
    base_test :     list
                la base de test
    classement :     list
                 les prévisions effectuées sur la base test
    résultat:
    ---------
    float
        un nombre décimal compris entre 0 et 1 
    """
    nbSucces = 0
    pass
        
def classe_majoritaire(ensemble,k):
    """
    classe les fleurs dans elements à partir de la base base_app
    selon l'algorithme du plus proche voisin
    
    paramètres:
    ----------
    ensemble :     list
                       les k plus proches voisins
    k :            int
                       le nombre de voisins
    résultat:
    ---------
    str
        la classe majoritaire parmi les k plus proches voisins
    """
    classes = ('Iris-setosa','Iris-versicolor','Iris-virginica')
    nb_classes = [0]*3
    #parcourir ensemble et compter les effectifs de chaque classe
    #nb_classes[0] étant l'effectif de la classe 'Iris-setosa', etc...
    pass
            
def insertion(d,b,d_min,b_min,k):
    """
    insertion de d et b au bon endroit dans les listes d_min et b_min
    
    paramètres:
    ----------
    d : float
             une distance
    b : list
             un élément de la base d'apprentissage
    d_min : list
             la liste des k plus petites distances
    b_min : list
             la liste des k plus proches voisins
    k     : int
            le nombre de voisins
                 
    résultat:
    ---------
    None
        met à jour d_min et b_min
    """
    d_min[k] = d
    b_min[k] = b
    #insertion de d et b au bon endroit dans les listes d_min et b_min
    for j in range(k,0,-1):
        pass
    
def kppv(base_app,elements,k):
    """
    classe les fleurs dans elements à partir de la base base_app
    selon l'algorithme des k  plus proches voisins
    
    paramètres:
    ----------
    base_app :     list
                la base d'apprentissage
    elements :     list
                 les fleurs à classer
    k     :        int
                 le nombre de voisins
    résultat:
    ---------
    list
        les fleurs ont été classées
    """
    elements_bis = deepcopy(elements)
    
    for i in range(len(elements)):
        d_min = [inf]*(k+1)
        b_min = [None]*(k+1)
        for b  in base_app:
            d = dist_eucli(elements[i][2],elements[i][3],b[2],b[3])
            insertion(d,b,d_min,b_min,k)
            #fin de l'insertion
        #recherche de la classe majoritaire dans b_min
        elements_bis[i][4] = classe_majoritaire(b_min,k)
    return elements_bis
    
def tracer(fleurs_app):
    x1 = []
    y1 = []
    x2 = []
    y2 = []
    x3 = []
    y3 = []
    coul = []
    for i in range(1,len(fleurs_app)):
        
        if fleurs_app[i][4] == 'Iris-setosa':
            x1.append(fleurs_app[i][2])
            y1.append(fleurs_app[i][3])
        elif fleurs_app[i][4] == 'Iris-versicolor':
            x2.append(fleurs_app[i][2])
            y2.append(fleurs_app[i][3])
        else:
            x3.append(fleurs_app[i][2])
            y3.append(fleurs_app[i][3])
    data = ((x1,y1), (x2,y2), (x3,y3))
    couleurs = ("green", "red", "blue")
    groupes = ("setosa", "versicolor", "virginica")
    
    fig = plt.figure()
    ax = fig.add_subplot(1, 1, 1)
    
    for donnee, coul, groupe in zip(data, couleurs, groupes):
        x, y = donnee
        ax.scatter(x, y, alpha=0.8, c=coul, edgecolors='none', s=30, label=groupe)
    
    plt.legend(loc=2)
    plt.xlabel('Longueur Pétale')
    plt.ylabel('Largeur Pétale')
    plt.show()
        
    
    
if __name__== "__main__":
    bases = []
    fleurs_app = []
    fleurs_test = []
    fleurs_classees = []
    fleurs_app = creationTableau2D("iris.csv")
    fleurs_app = mise_en_forme(fleurs_app)
    bases = creation_base_apprentissage_et_test(fleurs_app)
    
    fleurs_app = bases[0]
    fleurs_test = bases[1]
    
    #Test algorithme du plus proche voisins
     
    fleurs_classees = ppv(fleurs_app,fleurs_test)
    
    #Test algorithme des k plus proches voisins pour k = 3
    
    #k = 3
    #fleurs_classees = kppv(fleurs_app,fleurs_test,k)
    
    print("la mesure de performance vaut ",mesure_performance(fleurs_test,fleurs_classees))
    tracer(fleurs_app)